@@ -780,82 +780,119 @@ func testHelpAndVersionWithOptionsEnd(flag string, t *testing.T) {
 
 func TestHelpAndVersionWithOptionsEnd(t *testing.T) {
 	for _, flag := range []string{"-h", "--help", "-v", "--version"} {
-		testHelpAndVersionWithOptionsEnd(flag, t)
+		t.Run(flag, func(t *testing.T) {
+			testHelpAndVersionWithOptionsEnd(flag, t)
+		})
 	}
 }
 
 var genGolden = flag.Bool("g", false, "Generate golden file(s)")
 
 func TestHelpMessage(t *testing.T) {
-	var out, err string
-	defer captureAndRestoreOutput(&out, &err)()
-
-	exitCalled := false
-	defer exitShouldBeCalledWith(t, 0, &exitCalled)()
-
-	app := App("app", "App Desc")
-	app.Spec = "[-bdsuikqs] BOOL1 [STR1] INT3..."
-
-	// Options
-	app.Bool(BoolOpt{Name: "b bool1 u uuu", Value: false, EnvVar: "BOOL1", Desc: "Bool Option 1"})
-	app.Bool(BoolOpt{Name: "bool2", Value: true, EnvVar: " ", Desc: "Bool Option 2"})
-	app.Bool(BoolOpt{Name: "d", Value: true, EnvVar: "BOOL3", Desc: "Bool Option 3", HideValue: true})
-
-	app.String(StringOpt{Name: "s str1", Value: "", EnvVar: "STR1", Desc: "String Option 1"})
-	app.String(StringOpt{Name: "str2", Value: "a value", Desc: "String Option 2"})
-	app.String(StringOpt{Name: "v", Value: "another value", EnvVar: "STR3", Desc: "String Option 3", HideValue: true})
-
-	app.Int(IntOpt{Name: "i int1", Value: 0, EnvVar: "INT1 ALIAS_INT1"})
-	app.Int(IntOpt{Name: "int2", Value: 1, EnvVar: "INT2", Desc: "Int Option 2"})
-	app.Int(IntOpt{Name: "k", Value: 1, EnvVar: "INT3", Desc: "Int Option 3", HideValue: true})
-
-	app.Strings(StringsOpt{Name: "x strs1", Value: nil, EnvVar: "STRS1", Desc: "Strings Option 1"})
-	app.Strings(StringsOpt{Name: "strs2", Value: []string{"value1", "value2"}, EnvVar: "STRS2", Desc: "Strings Option 2"})
-	app.Strings(StringsOpt{Name: "z", Value: []string{"another value"}, EnvVar: "STRS3", Desc: "Strings Option 3", HideValue: true})
-
-	app.Ints(IntsOpt{Name: "q ints1", Value: nil, EnvVar: "INTS1", Desc: "Ints Option 1"})
-	app.Ints(IntsOpt{Name: "ints2", Value: []int{1, 2, 3}, EnvVar: "INTS2", Desc: "Ints Option 2"})
-	app.Ints(IntsOpt{Name: "j", Value: []int{1}, EnvVar: "INTS3", Desc: "Ints Option 3", HideValue: true})
-
-	// Args
-	app.Bool(BoolArg{Name: "BOOL1", Value: false, EnvVar: "BOOL1", Desc: "Bool Argument 1"})
-	app.Bool(BoolArg{Name: "BOOL2", Value: true, Desc: "Bool Argument 2"})
-	app.Bool(BoolArg{Name: "BOOL3", Value: true, EnvVar: "BOOL3", Desc: "Bool Argument 3", HideValue: true})
-
-	app.String(StringArg{Name: "STR1", Value: "", EnvVar: "STR1", Desc: "String Argument 1"})
-	app.String(StringArg{Name: "STR2", Value: "a value", EnvVar: "STR2", Desc: "String Argument 2"})
-	app.String(StringArg{Name: "STR3", Value: "another value", EnvVar: "STR3", Desc: "String Argument 3", HideValue: true})
-
-	app.Int(IntArg{Name: "INT1", Value: 0, EnvVar: "INT1", Desc: "Int Argument 1"})
-	app.Int(IntArg{Name: "INT2", Value: 1, EnvVar: "INT2", Desc: "Int Argument 2"})
-	app.Int(IntArg{Name: "INT3", Value: 1, EnvVar: "INT3", Desc: "Int Argument 3", HideValue: true})
-
-	app.Strings(StringsArg{Name: "STRS1", Value: nil, EnvVar: "STRS1", Desc: "Strings Argument 1"})
-	app.Strings(StringsArg{Name: "STRS2", Value: []string{"value1", "value2"}, EnvVar: "STRS2"})
-	app.Strings(StringsArg{Name: "STRS3", Value: []string{"another value"}, EnvVar: "STRS3", Desc: "Strings Argument 3", HideValue: true})
-
-	app.Ints(IntsArg{Name: "INTS1", Value: nil, EnvVar: "INTS1", Desc: "Ints Argument 1"})
-	app.Ints(IntsArg{Name: "INTS2", Value: []int{1, 2, 3}, EnvVar: "INTS2", Desc: "Ints Argument 2"})
-	app.Ints(IntsArg{Name: "INTS3", Value: []int{1}, EnvVar: "INTS3", Desc: "Ints Argument 3", HideValue: true})
-
-	app.Action = func() {}
-
-	app.Command("command1", "command1 description", func(cmd *Cmd) {})
-	app.Command("command2", "command2 description", func(cmd *Cmd) {})
-	app.Command("command3", "command3 description", func(cmd *Cmd) {})
-
-	require.NoError(t,
-		app.Run([]string{"app", "-h"}))
-
-	if *genGolden {
-		require.NoError(t,
-			ioutil.WriteFile("testdata/help-output.txt.golden", []byte(err), 0644))
+	cases := []struct {
+		name     string
+		params   []string
+		env      map[string]string
+		exitCode int
+	}{
+		{name: "top-help", params: []string{"app", "-h"}},
+		{name: "top-help-i-user", params: []string{"app", "-i=5"}, exitCode: 2},
+		{name: "top-help-i-env", params: []string{"app"}, env: map[string]string{"INT1": "25"}, exitCode: 2},
+		{name: "command1", params: []string{"app", "command1", "-h"}},
+		{name: "command2", params: []string{"app", "command2", "-h"}},
+		{name: "command3", params: []string{"app", "command3", "-h"}},
+		{name: "command3-child1", params: []string{"app", "command3", "child1", "-h"}},
+		{name: "command3-child2", params: []string{"app", "command3", "child2", "-h"}},
+		{name: "command4", params: []string{"app", "command4", "-h"}},
 	}
+	for _, cas := range cases {
+		cas := cas
+		t.Run(cas.name, func(t *testing.T) {
+			t.Logf("case: %+v", cas)
+			var out, stdErr string
+			defer captureAndRestoreOutput(&out, &stdErr)()
+			defer setAndRestoreEnv(cas.env)()
 
-	expected, e := ioutil.ReadFile("testdata/help-output.txt")
-	require.NoError(t, e, "Failed to read the expected help output from testdata/help-output.txt")
+			exitCalled := false
+			defer exitShouldBeCalledWith(t, cas.exitCode, &exitCalled)()
 
-	require.Equal(t, expected, []byte(err))
+			app := App("app", "App Desc")
+			app.Spec = "[-bdsuikqs] [BOOL1 STR1 INT3...]"
+
+			// Options
+			app.Bool(BoolOpt{Name: "b bool1 u uuu", Value: false, EnvVar: "BOOL1", Desc: "Bool Option 1"})
+			app.Bool(BoolOpt{Name: "bool2", Value: true, EnvVar: " ", Desc: "Bool Option 2"})
+			app.Bool(BoolOpt{Name: "d", Value: true, EnvVar: "BOOL3", Desc: "Bool Option 3", HideValue: true})
+
+			app.String(StringOpt{Name: "s str1", Value: "", EnvVar: "STR1", Desc: "String Option 1"})
+			app.String(StringOpt{Name: "str2", Value: "a value", Desc: "String Option 2"})
+			app.String(StringOpt{Name: "v", Value: "another value", EnvVar: "STR3", Desc: "String Option 3", HideValue: true})
+
+			app.Int(IntOpt{Name: "i int1", Value: 0, EnvVar: "INT1 ALIAS_INT1"})
+			app.Int(IntOpt{Name: "int2", Value: 1, EnvVar: "INT2", Desc: "Int Option 2"})
+			app.Int(IntOpt{Name: "k", Value: 1, EnvVar: "INT3", Desc: "Int Option 3", HideValue: true})
+
+			app.Strings(StringsOpt{Name: "x strs1", Value: nil, EnvVar: "STRS1", Desc: "Strings Option 1"})
+			app.Strings(StringsOpt{Name: "strs2", Value: []string{"value1", "value2"}, EnvVar: "STRS2", Desc: "Strings Option 2"})
+			app.Strings(StringsOpt{Name: "z", Value: []string{"another value"}, EnvVar: "STRS3", Desc: "Strings Option 3", HideValue: true})
+
+			app.Ints(IntsOpt{Name: "q ints1", Value: nil, EnvVar: "INTS1", Desc: "Ints Option 1"})
+			app.Ints(IntsOpt{Name: "ints2", Value: []int{1, 2, 3}, EnvVar: "INTS2", Desc: "Ints Option 2"})
+			app.Ints(IntsOpt{Name: "j", Value: []int{1}, EnvVar: "INTS3", Desc: "Ints Option 3", HideValue: true})
+
+			// Args
+			app.Bool(BoolArg{Name: "BOOL1", Value: false, EnvVar: "BOOL1", Desc: "Bool Argument 1"})
+			app.Bool(BoolArg{Name: "BOOL2", Value: true, Desc: "Bool Argument 2"})
+			app.Bool(BoolArg{Name: "BOOL3", Value: true, EnvVar: "BOOL3", Desc: "Bool Argument 3", HideValue: true})
+
+			app.String(StringArg{Name: "STR1", Value: "", EnvVar: "STR1", Desc: "String Argument 1"})
+			app.String(StringArg{Name: "STR2", Value: "a value", EnvVar: "STR2", Desc: "String Argument 2"})
+			app.String(StringArg{Name: "STR3", Value: "another value", EnvVar: "STR3", Desc: "String Argument 3", HideValue: true})
+
+			app.Int(IntArg{Name: "INT1", Value: 0, EnvVar: "INT1", Desc: "Int Argument 1"})
+			app.Int(IntArg{Name: "INT2", Value: 1, EnvVar: "INT2", Desc: "Int Argument 2"})
+			app.Int(IntArg{Name: "INT3", Value: 1, EnvVar: "INT3", Desc: "Int Argument 3", HideValue: true})
+
+			app.Strings(StringsArg{Name: "STRS1", Value: nil, EnvVar: "STRS1", Desc: "Strings Argument 1"})
+			app.Strings(StringsArg{Name: "STRS2", Value: []string{"value1", "value2"}, EnvVar: "STRS2"})
+			app.Strings(StringsArg{Name: "STRS3", Value: []string{"another value"}, EnvVar: "STRS3", Desc: "Strings Argument 3", HideValue: true})
+
+			app.Ints(IntsArg{Name: "INTS1", Value: nil, EnvVar: "INTS1", Desc: "Ints Argument 1"})
+			app.Ints(IntsArg{Name: "INTS2", Value: []int{1, 2, 3}, EnvVar: "INTS2", Desc: "Ints Argument 2"})
+			app.Ints(IntsArg{Name: "INTS3", Value: []int{1}, EnvVar: "INTS3", Desc: "Ints Argument 3", HideValue: true})
+
+			app.Command("command1", "command1 description", func(cmd *Cmd) {})
+			app.Command("command2", "command2 description", func(cmd *Cmd) {})
+			app.Command("command3", "command3 description", func(cmd *Cmd) {
+				cmd.Command("child1", "child1 description", func(cmd *Cmd) {
+					cmd.StringArg("ARG1", "", "arg1 desc")
+				})
+				cmd.Command("child2", "child2 description", func(cmd *Cmd) {
+					cmd.Hidden = true
+					cmd.StringOpt("o opt", "", "opt desc")
+				})
+			})
+			app.Command("command4", "command4 description", func(cmd *Cmd) {
+				cmd.Hidden = true
+			})
+
+			fmt.Printf("calling app with %+v\n", cas.params)
+			require.NoError(t,
+				app.Run(cas.params))
+
+			filename := fmt.Sprintf("testdata/help-output-%s.txt", cas.name)
+
+			if *genGolden {
+				require.NoError(t,
+					ioutil.WriteFile(filename, []byte(stdErr), 0644))
+			}
+
+			expected, e := ioutil.ReadFile(filename)
+			require.NoError(t, e, "Failed to read the expected help output from %s", filename)
+
+			require.Equal(t, string(expected), stdErr)
+		})
+	}
 }
 
 func TestLongHelpMessage(t *testing.T) {
@@ -2201,6 +2238,20 @@ func suppressOutput() func() {
 	return captureAndRestoreOutput(nil, nil)
 }
 
+func setAndRestoreEnv(env map[string]string) func() {
+	backup := map[string]string{}
+	for k, v := range env {
+		backup[k] = os.Getenv(k)
+		os.Setenv(k, v)
+	}
+
+	return func() {
+		for k, v := range backup {
+			os.Setenv(k, v)
+		}
+	}
+}
+
 func captureAndRestoreOutput(out, err *string) func() {
 	oldStdOut := stdOut
 	oldStdErr := stdErr
