@@ -12,7 +12,6 @@ import (
 	"github.com/jawher/mow.cli/internal/fsm"
 	"github.com/jawher/mow.cli/internal/lexer"
 	"github.com/jawher/mow.cli/internal/parser"
-	"github.com/jawher/mow.cli/internal/values"
 )
 
 /*
@@ -31,6 +30,8 @@ type Cmd struct {
 	Spec string
 	// The command long description to be shown when help is requested
 	LongDesc string
+	// Hide this command in the help messages
+	Hidden bool
 	// The command error handling strategy
 	ErrorHandling flag.ErrorHandling
 
@@ -544,7 +545,7 @@ func (c *Cmd) printHelp(longDesc bool) {
 		for _, arg := range c.args {
 			var (
 				env   = formatEnvVarsForHelp(arg.EnvVar)
-				value = formatValueForHelp(arg.HideValue, arg.Value)
+				value = formatValueForHelp(arg.HideValue, arg.DefaultValue)
 			)
 			printTabbedRow(w, arg.Name, joinStrings(arg.Desc, env, value))
 		}
@@ -557,21 +558,34 @@ func (c *Cmd) printHelp(longDesc bool) {
 			var (
 				optNames = formatOptNamesForHelp(opt)
 				env      = formatEnvVarsForHelp(opt.EnvVar)
-				value    = formatValueForHelp(opt.HideValue, opt.Value)
+				value    = formatValueForHelp(opt.HideValue, opt.DefaultValue)
 			)
 			printTabbedRow(w, optNames, joinStrings(opt.Desc, env, value))
 		}
 	}
 
-	if len(c.commands) > 0 {
+	commands := make([]*Cmd, 0, len(c.commands))
+	for _, c := range c.commands {
+		if err := c.doInit(); err != nil {
+			panic(err)
+		}
+
+		if c.Hidden {
+			continue
+		}
+
+		commands = append(commands, c)
+	}
+
+	if len(commands) > 0 {
 		fmt.Fprint(w, "\t\nCommands:\t\n")
 
-		for _, c := range c.commands {
+		for _, c := range commands {
 			fmt.Fprintf(w, "  %s\t%s\n", strings.Join(c.aliases, ", "), c.desc)
 		}
 	}
 
-	if len(c.commands) > 0 {
+	if len(commands) > 0 {
 		fmt.Fprintf(w, "\t\nRun '%s COMMAND --help' for more information on a command.\n", path)
 	}
 
@@ -604,18 +618,16 @@ func formatOptNamesForHelp(o *container.Container) string {
 	}
 }
 
-func formatValueForHelp(hide bool, v flag.Value) string {
+func formatValueForHelp(hide bool, v string) string {
 	if hide {
 		return ""
 	}
 
-	if dv, ok := v.(values.DefaultValued); ok {
-		if dv.IsDefault() {
-			return ""
-		}
+	if v == "" {
+		return ""
 	}
 
-	return fmt.Sprintf("(default %s)", v.String())
+	return fmt.Sprintf("(default %s)", v)
 }
 
 func formatEnvVarsForHelp(envVars string) string {
